@@ -12,12 +12,19 @@ except ImportError: #PY2
     from base64 import decodestring as decodebytes
 
 
-from tornado import web, escape
+from tornado import web
 
 from notebook.base.handlers import IPythonHandler
 
+
 class FilesHandler(IPythonHandler):
-    """serve files via ContentsManager"""
+    """serve files via ContentsManager
+
+    Normally used when ContentsManager is not a FileContentsManager.
+
+    FileContentsManager subclasses use AuthenticatedFilesHandler by default,
+    a subclass of StaticFileHandler.
+    """
 
     @web.authenticated
     def head(self, path):
@@ -26,10 +33,11 @@ class FilesHandler(IPythonHandler):
     @web.authenticated
     def get(self, path, include_body=True):
         cm = self.contents_manager
-        if cm.is_hidden(path):
+
+        if cm.is_hidden(path) and not cm.allow_hidden:
             self.log.info("Refusing to serve hidden file, via 404 Error")
             raise web.HTTPError(404)
-        
+
         path = path.strip('/')
         if '/' in path:
             _, name = path.rsplit('/', 1)
@@ -39,20 +47,22 @@ class FilesHandler(IPythonHandler):
         model = cm.get(path, type='file', content=include_body)
         
         if self.get_argument("download", False):
-            self.set_header('Content-Disposition','attachment; filename="%s"' % escape.url_escape(name))
+            self.set_attachment_header(name)
         
         # get mimetype from filename
         if name.endswith('.ipynb'):
             self.set_header('Content-Type', 'application/x-ipynb+json')
         else:
             cur_mime = mimetypes.guess_type(name)[0]
-            if cur_mime is not None:
+            if cur_mime == 'text/plain':
+                self.set_header('Content-Type', 'text/plain; charset=UTF-8')
+            elif cur_mime is not None:
                 self.set_header('Content-Type', cur_mime)
             else:
                 if model['format'] == 'base64':
                     self.set_header('Content-Type', 'application/octet-stream')
                 else:
-                    self.set_header('Content-Type', 'text/plain')
+                    self.set_header('Content-Type', 'text/plain; charset=UTF-8')
 
         if include_body:
             if model['format'] == 'base64':
@@ -64,6 +74,5 @@ class FilesHandler(IPythonHandler):
                 self.write(model['content'])
             self.flush()
 
-default_handlers = [
-    (r"/files/(.*)", FilesHandler),
-]
+
+default_handlers = []
