@@ -5,22 +5,24 @@
 // before all other objects so it exists when others register event handlers.
 // To register an event handler:
 //
-// require(['base/js/events'], function (events) {
+// requirejs(['base/js/events'], function (events) {
 //     events.on("event.Namespace", function () { do_stuff(); });
 // });
 
-define(['base/js/namespace', 'jquery'], function(IPython, $) {
+define(['jquery', 'base/js/namespace'], function($, Jupyter) {
     "use strict";
-
-    var Events = function () {};
     
-    var events = new Events();
+    // Events singleton
+    if (!window._Events) {
+        window._Events = function () {};
+        window._events = new window._Events();
+    }
     
     // Backwards compatability.
-    IPython.Events = Events;
-    IPython.events = events;
+    Jupyter.Events = window._Events;
+    Jupyter.events = window._events;
     
-    var events = $([events]);
+    var events = $([window._events]);
 
     // catch and log errors in triggered events
     events._original_trigger = events.trigger;
