@@ -169,20 +169,27 @@ define([
 
     // Shortcut manager class
 
-    var ShortcutManager = function (delay, events, actions, env) {
+    var ShortcutManager = function (delay, events, actions, env, config, mode) {
         /**
          * A class to deal with keyboard event and shortcut
          *
          * @class ShortcutManager
          * @constructor
+         *
+         * :config: configobjet on which to call `update(....)` to persist the config.
+         * :mode: mode of this shortcut manager where to persist config.
          */
+        mode = mode || 'command';
         this._shortcuts = {};
+        this._defaults_bindings = [];
         this.delay = delay || 800; // delay in milliseconds
         this.events = events;
         this.actions = actions;
         this.actions.extend_env(env);
         this._queue = [];
         this._cleartimeout = null;
+        this._config = config;
+        this._mode = mode;
         Object.seal(this);
     };
 
@@ -225,10 +232,21 @@ define([
         }
         return dct;
     };
+
+
+    ShortcutManager.prototype.get_action_shortcuts = function(name){
+      var ftree = flatten_shorttree(this._shortcuts);
+      var res = [];
+      for (var sht in ftree ){
+        if(ftree[sht] === name){
+          res.push(sht);
+        }
+      }
+      return res;
+    };
     
     ShortcutManager.prototype.get_action_shortcut = function(name){
       var ftree = flatten_shorttree(this._shortcuts);
-      var res = {};
       for (var sht in ftree ){
         if(ftree[sht] === name){
           return sht;
@@ -325,7 +343,7 @@ define([
             if(typeof(current_node) === 'string'){
                 delete tree[shortcut_array[0]];
             } else {
-                throw('try to delete non-leaf');
+                throw new Error('try to delete non-leaf');
             }
         } else {
             this._remove_leaf(shortcut_array.slice(1),  tree[shortcut_array[0]], allow_node);
@@ -335,8 +353,30 @@ define([
         }
     };
 
+    ShortcutManager.prototype.is_available_shortcut = function(shortcut){
+        var shortcut_array = shortcut.split(',');
+        return this._is_available_shortcut(shortcut_array, this._shortcuts);
+    };
+
+    ShortcutManager.prototype._is_available_shortcut = function(shortcut_array, tree){
+        var current_node = tree[shortcut_array[0]];
+        if(!shortcut_array[0]){
+            return false;
+        }
+        if(current_node === undefined){
+            return true;
+        } else {
+            if (typeof(current_node) == 'string'){
+                return false;
+            } else { // assume is a sub-shortcut tree
+                return this._is_available_shortcut(shortcut_array.slice(1), current_node);
+            }
+        }
+    };
+
     ShortcutManager.prototype._set_leaf = function(shortcut_array, action_name, tree){
         var current_node = tree[shortcut_array[0]];
+
         if(shortcut_array.length === 1){
             if(current_node !== undefined && typeof(current_node) !== 'string'){
                 console.warn('[warning], you are overriting a long shortcut with a shorter one');
@@ -357,9 +397,55 @@ define([
         }
     };
 
+    ShortcutManager.prototype._persist_shortcut = function(shortcut, data) {
+        /**
+         * add a shortcut to this manager and persist it to the config file. 
+         **/ 
+        shortcut = shortcut.toLowerCase();
+        this.add_shortcut(shortcut, data);
+        var patch = {keys:{}};
+        var b = {bind:{}};
+        patch.keys[this._mode] = {bind:{}};
+        patch.keys[this._mode].bind[shortcut] = data;
+        this._config.update(patch);
+    };
+
+    ShortcutManager.prototype._persist_remove_shortcut = function(shortcut){
+        /**
+         * Remove a shortcut from this manager and persist its removal.
+         */
+
+        shortcut = shortcut.toLowerCase();
+        this.remove_shortcut(shortcut);
+        var patch = {keys: {}};
+        var b = {bind: {}};
+        patch.keys[this._mode] = {bind:{}};
+        patch.keys[this._mode].bind[shortcut] = null;
+        this._config.update(patch);
+
+        // if the shortcut we unbind is a default one, we add it to the list of
+        // things to unbind at startup
+        if( this._defaults_bindings.indexOf(shortcut) !== -1 ){
+            var cnf = (this._config.data.keys || {})[this._mode];
+            var unbind_array = cnf.unbind || [];
+
+
+            // unless it's already there (like if we have remapped a default
+            // shortcut to another command): unbind it)
+            if(unbind_array.indexOf(shortcut) === -1){
+                var _parray = unbind_array.concat(shortcut);
+                var unbind_patch = {keys:{}};
+                unbind_patch.keys[this._mode] = {unbind:_parray};
+                console.warn('up:', unbind_patch);
+                this._config.update(unbind_patch);
+            }
+        }
+    };
+
+
     ShortcutManager.prototype.add_shortcut = function (shortcut, data, suppress_help_update) {
         /**
-         * Add a action to be handled by shortcut manager. 
+         * Add an action to be handled by shortcut manager. 
          * 
          * - `shortcut` should be a `Shortcut Sequence` of the for `Ctrl-Alt-C,Meta-X`...
          * - `data` could be an `action name`, an `action` or a `function`.
@@ -370,8 +456,8 @@ define([
         if (! action_name){
           throw new Error('does not know how to deal with : ' + data);
         }
-        shortcut = normalize_shortcut(shortcut);
-        this.set_shortcut(shortcut, action_name);
+        var _shortcut = normalize_shortcut(shortcut);
+        this.set_shortcut(_shortcut, action_name);
 
         if (!suppress_help_update) {
             // update the keyboard shortcuts notebook help
@@ -392,11 +478,25 @@ define([
         this.events.trigger('rebuild.QuickHelp');
     };
 
+    ShortcutManager.prototype._add_default_shortcuts = function (data) {
+        /**
+         * same as add_shortcuts, but register them as "default" that if persistently unbound, with
+         * persist_remove_shortcut, need to be on the "unbind" list. 
+         **/
+        this._defaults_bindings = this._defaults_bindings.concat(Object.keys(data));
+        this.add_shortcuts(data);
+
+    };
+
     ShortcutManager.prototype.remove_shortcut = function (shortcut, suppress_help_update) {
         /**
          * Remove the binding of shortcut `sortcut` with its action.
          * throw an error if trying to remove a non-exiting shortcut
          **/
+        if(!shortcut){
+            console.warn('trying to remove empty shortcut');
+            return;
+        }
         shortcut = normalize_shortcut(shortcut);
         if( typeof(shortcut) === 'string'){
             shortcut = shortcut.split(',');
@@ -405,15 +505,14 @@ define([
          *  The shortcut error should be explicit here, because it will be
          *  seen by users.
          */
-        try
-        {
+        try {
           this._remove_leaf(shortcut, this._shortcuts);
           if (!suppress_help_update) {
             // update the keyboard shortcuts notebook help
             this.events.trigger('rebuild.QuickHelp');
           }
         } catch (ex) {
-          throw new Error('trying to remove a non-existent shortcut', shortcut);
+          throw new Error('trying to remove a non-existent shortcut', shortcut, typeof shortcut);
         }
     };
 
