@@ -5,9 +5,10 @@ define([
     'jquery',
     'codemirror/lib/codemirror',
     'moment',
+    'underscore',
     // silently upgrades CodeMirror
     'codemirror/mode/meta',
-], function($, CodeMirror, moment){
+], function($, CodeMirror, moment, _){
     "use strict";
     
     // keep track of which extensions have been loaded already
@@ -34,7 +35,7 @@ define([
             requirejs([ext_path], function(module) {
                 if (!is_loaded(extension)) {
                     console.log("Loading extension: " + extension);
-                    if (module.load_ipython_extension) {
+                    if (module && module.load_ipython_extension) {
                         Promise.resolve(module.load_ipython_extension()).then(function() {
                             resolve(module);
                         }).catch(reject);
@@ -56,7 +57,7 @@ define([
      * @return {Promise} that resolves to a list of loaded module handles.
      */
     var load_extensions = function () {
-        console.log("load_extensions", arguments);
+        console.log('load_extensions', arguments);
         return Promise.all(Array.prototype.map.call(arguments, load_extension)).catch(function(err) {
             console.error("Failed to load extension" + (err.requireModules.length>1?'s':'') + ":", err.requireModules, err);
         });
@@ -64,8 +65,8 @@ define([
 
     /**
      * Return a list of extensions that should be active
-     * The config for nbextensions comes in as a dict where keys are 
-     * nbextensions paths and the values are a bool indicating if it 
+     * The config for nbextensions comes in as a dict where keys are
+     * nbextensions paths and the values are a bool indicating if it
      * should be active. This returns a list of nbextension paths
      * where the value is true
      */
@@ -218,165 +219,257 @@ define([
         return uuid;
     };
 
-
-    //Fix raw text to parse correctly in crazy XML
-    function xmlencode(string) {
-        return string.replace(/\&/g,'&'+'amp;')
-            .replace(/</g,'&'+'lt;')
-            .replace(/>/g,'&'+'gt;')
-            .replace(/\'/g,'&'+'apos;')
-            .replace(/\"/g,'&'+'quot;')
-            .replace(/`/g,'&'+'#96;');
-    }
-
-
-    //Map from terminal commands to CSS classes
-    var ansi_colormap = {
-        "01":"ansibold",
-        
-        "30":"ansiblack",
-        "31":"ansired",
-        "32":"ansigreen",
-        "33":"ansiyellow",
-        "34":"ansiblue",
-        "35":"ansipurple",
-        "36":"ansicyan",
-        "37":"ansigray",
-        
-        "40":"ansibgblack",
-        "41":"ansibgred",
-        "42":"ansibggreen",
-        "43":"ansibgyellow",
-        "44":"ansibgblue",
-        "45":"ansibgpurple",
-        "46":"ansibgcyan",
-        "47":"ansibggray"
-    };
+    var _ANSI_COLORS = [
+        "ansi-black",
+        "ansi-red",
+        "ansi-green",
+        "ansi-yellow",
+        "ansi-blue",
+        "ansi-magenta",
+        "ansi-cyan",
+        "ansi-white",
+        "ansi-black-intense",
+        "ansi-red-intense",
+        "ansi-green-intense",
+        "ansi-yellow-intense",
+        "ansi-blue-intense",
+        "ansi-magenta-intense",
+        "ansi-cyan-intense",
+        "ansi-white-intense",
+    ];
     
-    function _process_numbers(attrs, numbers) {
-        // process ansi escapes
-        var n = numbers.shift();
-        if (ansi_colormap[n]) {
-            if ( ! attrs["class"] ) {
-                attrs["class"] = ansi_colormap[n];
+    function _pushColoredChunk(chunk, fg, bg, bold, underline, inverse, out) {
+        if (chunk) {
+            var classes = [];
+            var styles = [];
+
+            if (bold && typeof fg === "number" && 0 <= fg && fg < 8) {
+                fg += 8;  // Bold text uses "intense" colors
+            }
+            if (inverse) {
+                [fg, bg] = [bg, fg];
+            }
+
+            if (typeof fg === "number") {
+                classes.push(_ANSI_COLORS[fg] + "-fg");
+            } else if (fg.length) {
+                styles.push("color: rgb(" + fg + ")");
+            } else if (inverse) {
+                classes.push("ansi-default-inverse-fg");
+            }
+
+            if (typeof bg === "number") {
+                classes.push(_ANSI_COLORS[bg] + "-bg");
+            } else if (bg.length) {
+                styles.push("background-color: rgb(" + bg + ")");
+            } else if (inverse) {
+                classes.push("ansi-default-inverse-bg");
+            }
+
+            if (bold) {
+                classes.push("ansi-bold");
+            }
+
+            if (underline) {
+                classes.push("ansi-underline");
+            }
+
+            if (classes.length || styles.length) {
+                out.push("<span");
+                if (classes.length) {
+                    out.push(' class="' + classes.join(" ") + '"');
+                }
+                if (styles.length) {
+                    out.push(' style="' + styles.join("; ") + '"');
+                }
+                out.push(">");
+                out.push(chunk);
+                out.push("</span>");
             } else {
-                attrs["class"] += " " + ansi_colormap[n];
-            }
-        } else if (n == "38" || n == "48") {
-            // VT100 256 color or 24 bit RGB
-            if (numbers.length < 2) {
-                console.log("Not enough fields for VT100 color", numbers);
-                return;
-            }
-            
-            var index_or_rgb = numbers.shift();
-            var r,g,b;
-            if (index_or_rgb == "5") {
-                // 256 color
-                var idx = parseInt(numbers.shift(), 10);
-                if (idx < 16) {
-                    // indexed ANSI
-                    // ignore bright / non-bright distinction
-                    idx = idx % 8;
-                    var ansiclass = ansi_colormap[n[0] + (idx % 8).toString()];
-                    if ( ! attrs["class"] ) {
-                        attrs["class"] = ansiclass;
-                    } else {
-                        attrs["class"] += " " + ansiclass;
-                    }
-                    return;
-                } else if (idx < 232) {
-                    // 216 color 6x6x6 RGB
-                    idx = idx - 16;
-                    b = idx % 6;
-                    g = Math.floor(idx / 6) % 6;
-                    r = Math.floor(idx / 36) % 6;
-                    // convert to rgb
-                    r = (r * 51);
-                    g = (g * 51);
-                    b = (b * 51);
-                } else {
-                    // grayscale
-                    idx = idx - 231;
-                    // it's 1-24 and should *not* include black or white,
-                    // so a 26 point scale
-                    r = g = b = Math.floor(idx * 256 / 26);
-                }
-            } else if (index_or_rgb == "2") {
-                // Simple 24 bit RGB
-                if (numbers.length > 3) {
-                    console.log("Not enough fields for RGB", numbers);
-                    return;
-                }
-                r = numbers.shift();
-                g = numbers.shift();
-                b = numbers.shift();
-            } else {
-                console.log("unrecognized control", numbers);
-                return;
-            }
-            if (r !== undefined) {
-                // apply the rgb color
-                var line;
-                if (n == "38") {
-                    line = "color: ";
-                } else {
-                    line = "background-color: ";
-                }
-                line = line + "rgb(" + r + "," + g + "," + b + ");";
-                if ( !attrs.style ) {
-                    attrs.style = line;
-                } else {
-                    attrs.style += " " + line;
-                }
+                out.push(chunk);
             }
         }
     }
 
-    function _ansispan(str) {
-        // ansispan function adapted from github.com/mmalecki/ansispan (MIT License)
-        // regular ansi escapes (using the table above)
-        var is_open = false;
-        return str.replace(/\033\[(0?[01]|22|39)?([;\d]+)?m/g, function(match, prefix, pattern) {
-            if (!pattern || prefix === '39') {
-                // [(01|22|39|)m close spans
-                if (is_open) {
-                    is_open = false;
-                    return "</span>";
-                } else {
-                    return "";
-                }
-            } else {
-                is_open = true;
-
-                // consume sequence of color escapes
-                var numbers = pattern.match(/\d+/g);
-                var attrs = {};
-                while (numbers.length > 0) {
-                    _process_numbers(attrs, numbers);
-                }
-
-                var span = "<span ";
-                Object.keys(attrs).map(function (attr) {
-                    span = span + " " + attr + '="' + attrs[attr] + '"';
-                });
-                return span + ">";
+    function _getExtendedColors(numbers) {
+        var r, g, b;
+        var n = numbers.shift();
+        if (n === 2 && numbers.length >= 3) {
+            // 24-bit RGB
+            r = numbers.shift();
+            g = numbers.shift();
+            b = numbers.shift();
+            if ([r, g, b].some(function (c) { return c < 0 || 255 < c; })) {
+                throw new RangeError("Invalid range for RGB colors");
             }
-        });
+        } else if (n === 5 && numbers.length >= 1) {
+            // 256 colors
+            var idx = numbers.shift();
+            if (idx < 0) {
+                throw new RangeError("Color index must be >= 0");
+            } else if (idx < 16) {
+                // 16 default terminal colors
+                return idx;
+            } else if (idx < 232) {
+                // 6x6x6 color cube, see https://stackoverflow.com/a/27165165/500098
+                r = Math.floor((idx - 16) / 36);
+                r = r > 0 ? 55 + r * 40 : 0;
+                g = Math.floor(((idx - 16) % 36) / 6);
+                g = g > 0 ? 55 + g * 40 : 0;
+                b = (idx - 16) % 6;
+                b = b > 0 ? 55 + b * 40 : 0;
+            } else if (idx < 256) {
+                // grayscale, see https://stackoverflow.com/a/27165165/500098
+                r = g = b = (idx - 232) * 10 + 8;
+            } else {
+                throw new RangeError("Color index must be < 256");
+            }
+        } else {
+            throw new RangeError("Invalid extended color specification");
+        }
+        return [r, g, b];
     }
 
-    // Transform ANSI color escape codes into HTML <span> tags with css
-    // classes listed in the above ansi_colormap object. The actual color used
-    // are set in the css file.
-    function fixConsole(txt) {
-        txt = xmlencode(txt);
+    function _ansispan(str) {
+        var ansi_re = /\x1b\[(.*?)([@-~])/g;
+        var fg = [];
+        var bg = [];
+        var bold = false;
+        var underline = false;
+        var inverse = false;
+        var match;
+        var out = [];
+        var numbers = [];
+        var start = 0;
 
-        // Strip all ANSI codes that are not color related.  Matches
-        // all ANSI codes that do not end with "m".
-        var ignored_re = /(?=(\033\[[?\d;=]*[a-ln-zA-Z]{1}))\1(?!m)/g;
-        txt = txt.replace(ignored_re, "");
-        
-        // color ansi codes
+        str += "\x1b[m";  // Ensure markup for trailing text
+        while ((match = ansi_re.exec(str))) {
+            if (match[2] === "m") {
+                var items = match[1].split(";");
+                for (var i = 0; i < items.length; i++) {
+                    var item = items[i];
+                    if (item === "") {
+                        numbers.push(0);
+                    } else if (item.search(/^\d+$/) !== -1) {
+                        numbers.push(parseInt(item));
+                    } else {
+                        // Ignored: Invalid color specification
+                        numbers.length = 0;
+                        break;
+                    }
+                }
+            } else {
+                // Ignored: Not a color code
+            }
+            var chunk = str.substring(start, match.index);
+	    _pushColoredChunk(chunk, fg, bg, bold, underline, inverse, out);
+            start = ansi_re.lastIndex;
+
+            while (numbers.length) {
+                var n = numbers.shift();
+                switch (n) {
+                    case 0:
+                        fg = bg = [];
+                        bold = false;
+                        underline = false;
+                        inverse = false;
+                        break;
+                    case 1:
+                    case 5:
+                        bold = true;
+                        break;
+                    case 4:
+                        underline = true;
+                        break;
+                    case 7:
+                        inverse = true;
+                        break;
+                    case 21:
+                    case 22:
+                        bold = false;
+                        break;
+                    case 24:
+                        underline = false;
+                        break;
+                    case 27:
+                        inverse = false;
+                        break;
+                    case 30:
+                    case 31:
+                    case 32:
+                    case 33:
+                    case 34:
+                    case 35:
+                    case 36:
+                    case 37:
+                        fg = n - 30;
+                        break;
+                    case 38:
+                        try {
+                            fg = _getExtendedColors(numbers);
+                        } catch(e) {
+                            numbers.length = 0;
+                        }
+                        break;
+                    case 39:
+                        fg = [];
+                        break;
+                    case 40:
+                    case 41:
+                    case 42:
+                    case 43:
+                    case 44:
+                    case 45:
+                    case 46:
+                    case 47:
+                        bg = n - 40;
+                        break;
+                    case 48:
+                        try {
+                            bg = _getExtendedColors(numbers);
+                        } catch(e) {
+                            numbers.length = 0;
+                        }
+                        break;
+                    case 49:
+                        bg = [];
+                        break;
+		    case 90:
+		    case 91:
+		    case 92:
+		    case 93:
+		    case 94:
+		    case 95:
+		    case 96:
+		    case 97:
+			fg = n - 90 + 8;
+                        break;
+		    case 100:
+		    case 101:
+		    case 102:
+		    case 103:
+		    case 104:
+		    case 105:
+		    case 106:
+		    case 107:
+			bg = n - 100 + 8;
+                        break;
+                    default:
+                        // Unknown codes are ignored
+                }
+            }
+        }
+        return out.join("");
+    }
+
+    // Transform ANSI color escape codes into HTML <span> tags with CSS
+    // classes such as "ansi-green-intense-fg".
+    // The actual colors used are set in the CSS file.
+    // This is supposed to have the same behavior as nbconvert.filters.ansi2html()
+    function fixConsole(txt) {
+        txt = _.escape(txt);
+
+        // color ansi codes (and remove non-color escape sequences)
         txt = _ansispan(txt);
         return txt;
     }
@@ -385,16 +478,32 @@ define([
     // carriage return characters
     function fixCarriageReturn(txt) {
         txt = txt.replace(/\r+\n/gm, '\n'); // \r followed by \n --> newline
-        while (txt.search(/\r/g) > -1) {
-            var base = txt.match(/^.*\r+/m)[0].replace(/\r/, '');
-            var insert = txt.match(/\r+.*$/m)[0].replace(/\r/, '');
+        while (txt.search(/\r[^$]/g) > -1) {
+            var base = txt.match(/^(.*)\r+/m)[1];
+            var insert = txt.match(/\r+(.*)$/m)[1];
             insert = insert + base.slice(insert.length, base.length);
-            txt = txt.replace(/\r+.*$/m, '\r').replace(/^.*\r+/m, insert);
+            txt = txt.replace(/\r+.*$/m, '\r').replace(/^.*\r/m, insert);
         }
         return txt;
     }
 
-    // Locate any URLs and convert them to a anchor tag
+    // Remove characters that are overridden by backspace characters
+    function fixBackspace(txt) {
+        var tmp = txt;
+        do {
+            txt = tmp;
+            // Cancel out anything-but-newline followed by backspace
+            tmp = txt.replace(/[^\n]\x08/gm, '');
+        } while (tmp.length < txt.length);
+        return txt;
+    }
+
+    // Remove characters overridden by backspace and carriage return
+    function fixOverwrittenChars(txt) {
+        return fixCarriageReturn(fixBackspace(txt));
+    }
+
+    // Locate any URLs and convert them to an anchor tag
     function autoLinkUrls(txt) {
         return txt.replace(/(^|\s)(https?|ftp)(:[^'"<>\s]+)/gi,
             "$1<a target=\"_blank\" href=\"$2$3\">$2$3</a>");
@@ -416,7 +525,7 @@ define([
          * wrapper around contructor to avoid requiring `var a = new constructor()`
          * useful for passing constructors as callbacks,
          * not for programmer laziness.
-         * from http://programmers.stackexchange.com/questions/118798
+         * from https://programmers.stackexchange.com/questions/118798
          */
         return function () {
             var obj = Object.create(constructor.prototype);
@@ -520,14 +629,14 @@ define([
          * until we are building an actual request
          */
         var val = $('body').data(key);
-        if (!val)
+        if (typeof val === 'undefined')
             return val;
         return decodeURIComponent(val);
     };
     
     var to_absolute_cursor_pos = function (cm, cursor) {
         console.warn('`utils.to_absolute_cursor_pos(cm, pos)` is deprecated. Use `cm.indexFromPos(cursor)`');
-        return cm.indexFromPos(cusrsor);
+        return cm.indexFromPos(cursor);
     };
     
     var from_absolute_cursor_pos = function (cm, cursor_pos) {
@@ -535,7 +644,7 @@ define([
         return cm.posFromIndex(cursor_pos);
     };
     
-    // http://stackoverflow.com/questions/2400935/browser-detection-in-javascript
+    // https://stackoverflow.com/questions/2400935/browser-detection-in-javascript
     var browser = (function() {
         if (typeof navigator === 'undefined') {
             // navigator undefined in node
@@ -548,7 +657,7 @@ define([
         return M;
     })();
 
-    // http://stackoverflow.com/questions/11219582/how-to-detect-my-browser-version-and-operating-system-using-javascript
+    // https://stackoverflow.com/questions/11219582/how-to-detect-my-browser-version-and-operating-system-using-javascript
     var platform = (function () {
         if (typeof navigator === 'undefined') {
             // navigator undefined in node
@@ -564,7 +673,7 @@ define([
     
     var get_url_param = function (name) {
         // get a URL parameter. I cannot believe we actually need this.
-        // Based on http://stackoverflow.com/a/25359264/938949
+        // Based on https://stackoverflow.com/a/25359264/938949
         var match = new RegExp('[?&]' + name + '=([^&]*)').exec(window.location.search);
         if (match){
             return decodeURIComponent(match[1] || '');
@@ -649,7 +758,7 @@ define([
             CodeMirror.findModeByMIME(modename) ||
             {mode: modename, mime: modename};
 
-        require([
+        requirejs([
                 // might want to use CodeMirror.modeURL here
                 ['codemirror/mode', info.mode, info.mode].join('/'),
             ], function() {
@@ -676,6 +785,41 @@ define([
         return wrapped_error;
     };
     
+    var ajax = function (url, settings) {
+        // like $.ajax, but ensure XSRF or Authorization header is set
+        if (typeof url === "object") {
+            // called with single argument: $.ajax({url: '...'})
+            settings = url;
+            url = settings.url;
+            delete settings.url;
+        }
+        settings = _add_auth_header(settings);
+        return $.ajax(url, settings);
+    };
+    
+    var _get_cookie = function (name) {
+        // from tornado docs: http://www.tornadoweb.org/en/stable/guide/security.html
+        var r = document.cookie.match("\\b" + name + "=([^;]*)\\b");
+        return r ? r[1] : undefined;
+    }
+
+    var _add_auth_header = function (settings) {
+        /**
+         * Adds auth header to jquery ajax settings
+         */
+        settings = settings || {};
+        if (!settings.headers) {
+            settings.headers = {};
+        }
+        if (!settings.headers.Authorization) {
+            var xsrf_token = _get_cookie('_xsrf');
+            if (xsrf_token) {
+                settings.headers['X-XSRFToken'] = xsrf_token;
+            }
+        }
+        return settings;
+    };
+
     var promising_ajax = function(url, settings) {
         /**
          * Like $.ajax, but returning an ES6 promise. success and error settings
@@ -690,7 +834,7 @@ define([
                 log_ajax_error(jqXHR, status, error);
                 reject(wrap_ajax_error(jqXHR, status, error));
             };
-            $.ajax(url, settings);
+            ajax(url, settings);
         });
     };
 
@@ -736,7 +880,7 @@ define([
 
             // Try loading the view module using require.js
             if (module_name) {
-                require([module_name], function(module) {
+                requirejs([module_name], function(module) {
                     if (module[class_name] === undefined) {
                         reject(new Error('Class '+class_name+' not found in module '+module_name));
                     } else {
@@ -810,11 +954,40 @@ define([
         if(!window.MathJax){
             return;
         }
-        return $el.map(function(){
+        $el.map(function(){
             // MathJax takes a DOM node: $.map makes `this` the context
-            return MathJax.Hub.Queue(["Typeset", MathJax.Hub, this]);
+            MathJax.Hub.Queue(["Typeset", MathJax.Hub, this]);
+            try {
+                MathJax.Hub.Queue(
+                    ["Require", MathJax.Ajax, "[MathJax]/extensions/TeX/AMSmath.js"],
+                    function() { MathJax.InputJax.TeX.resetEquationNumbers(); }
+                );
+            } catch (e) {
+                console.error("Error queueing resetEquationNumbers:", e);
+            }
         });
     };
+
+    var parse_b64_data_uri = function(uri) {
+        /**
+         * Parses a base64 encoded data-uri to extract mimetype and the
+         * base64 string.
+         *
+         * For example, given 'data:image/png;base64,iVBORw', it will return
+         * ["image/png", "iVBORw"]
+         *
+         * Parameters
+         */
+        // For performance reasons, the non-greedy ? qualifiers are crucial so
+        // that the matcher stops early on big blobs. Without them, it will try
+        // to match the whole blob which can take ages
+        var regex = /^data:(.+?\/.+?);base64,/;
+        var matches = uri.match(regex);
+        var mime = matches[1];
+        // matches[0] contains the whole data-uri prefix
+        var b64_data = uri.slice(matches[0].length);
+        return [mime, b64_data];
+    };
     
     var time = {};
     time.milliseconds = {};
@@ -850,8 +1023,118 @@ define([
             return time.milliseconds.h;
         }
     };
+
+    var format_datetime = function(date) {
+        var text = moment(date).fromNow();
+        return text === 'a few seconds ago' ? 'seconds ago' : text;
+    };
+
+    var datetime_sort_helper = function(a, b, order) {
+        if (moment(a).isBefore(moment(b))) {
+            return (order == 1) ? -1 : 1;
+        } else if (moment(a).isSame(moment(b))) {
+            return 0;
+        } else {
+            return (order == 1) ? 1 : -1;
+        }
+    };
+
+
+    // javascript stores text as utf16 and string indices use "code units",
+    // which stores high-codepoint characters as "surrogate pairs",
+    // which occupy two indices in the javascript string.
+    // We need to translate cursor_pos in the protocol (in characters)
+    // to js offset (with surrogate pairs taking two spots).
+    function js_idx_to_char_idx (js_idx, text) {
+        var char_idx = js_idx;
+        for (var i = 0; i + 1 < text.length && i < js_idx; i++) {
+            var char_code = text.charCodeAt(i);
+            // check for surrogate pair
+            if (char_code >= 0xD800 && char_code <= 0xDBFF) {
+                var next_char_code = text.charCodeAt(i+1);
+                if (next_char_code >= 0xDC00 && next_char_code <= 0xDFFF) {
+                    char_idx--;
+                    i++;
+                }
+            }
+        }
+        return char_idx;
+    }
+
+    function char_idx_to_js_idx (char_idx, text) {
+        var js_idx = char_idx;
+        for (var i = 0; i + 1 < text.length && i < js_idx; i++) {
+            var char_code = text.charCodeAt(i);
+            // check for surrogate pair
+            if (char_code >= 0xD800 && char_code <= 0xDBFF) {
+                var next_char_code = text.charCodeAt(i+1);
+                if (next_char_code >= 0xDC00 && next_char_code <= 0xDFFF) {
+                    js_idx++;
+                    i++;
+                }
+            }
+        }
+        return js_idx;
+    }
+
+    if ('𝐚'.length === 1) {
+        // If javascript fixes string indices of non-BMP characters,
+        // don't keep shifting offsets to compensate for surrogate pairs
+        char_idx_to_js_idx = js_idx_to_char_idx = function (idx, text) { return idx; };
+    }
+
+    // Test if a drag'n'drop event contains a file (as opposed to an HTML
+    // element/text from the document)
+    var dnd_contain_file = function(event) {
+        // As per the HTML5 drag'n'drop spec, the dataTransfer.types should
+        // contain one "Files" type if a file is being dragged
+        // https://www.w3.org/TR/2011/WD-html5-20110113/dnd.html#dom-datatransfer-types
+        if (event.dataTransfer.types) {
+            for (var i = 0; i < event.dataTransfer.types.length; i++) {
+                if (event.dataTransfer.types[i] == "Files") {
+                    return true;
+                }
+            }
+        }
+        return false;
+    };
+
+    var throttle = function(fn, time) {
+      var pending = null;
+
+      return function () {
+        if (pending) return;
+        pending = setTimeout(run, time);
+
+        return function () {
+          clearTimeout(pending);
+          pending = null;
+        }
+      }
+
+      function run () {
+        pending = null;
+        fn();
+      }
+    }
     
+    var change_favicon = function (src) {
+        var link = document.createElement('link'),
+            oldLink = document.getElementById('favicon');
+        link.id = 'favicon';
+        link.type = 'image/x-icon';
+        link.rel = 'shortcut icon';
+        link.href = utils.url_path_join(utils.get_body_data('baseUrl'), src);
+        if (oldLink && (link.href === oldLink.href)) {
+            // This favicon is already set, don't modify the DOM.
+            return;
+        }
+        if (oldLink) document.head.removeChild(oldLink);
+        document.head.appendChild(link);
+    };
+
     var utils = {
+        throttle: throttle,
         is_loaded: is_loaded,
         load_extension: load_extension,
         load_extensions: load_extensions,
@@ -861,6 +1144,8 @@ define([
         uuid : uuid,
         fixConsole : fixConsole,
         fixCarriageReturn : fixCarriageReturn,
+        fixBackspace : fixBackspace,
+        fixOverwrittenChars: fixOverwrittenChars,
         autoLinkUrls : autoLinkUrls,
         points_to_pixels : points_to_pixels,
         get_body_data : get_body_data,
@@ -880,10 +1165,11 @@ define([
         is_or_has : is_or_has,
         is_focused : is_focused,
         mergeopt: mergeopt,
-        ajax_error_msg : ajax_error_msg,
-        log_ajax_error : log_ajax_error,
         requireCodeMirrorMode : requireCodeMirrorMode,
         XHR_ERROR : XHR_ERROR,
+        ajax : ajax,
+        ajax_error_msg : ajax_error_msg,
+        log_ajax_error : log_ajax_error,
         wrap_ajax_error : wrap_ajax_error,
         promising_ajax : promising_ajax,
         WrappedError: WrappedError,
@@ -891,8 +1177,15 @@ define([
         resolve_promises_dict: resolve_promises_dict,
         reject: reject,
         typeset: typeset,
+        parse_b64_data_uri: parse_b64_data_uri,
         time: time,
-        _ansispan:_ansispan
+        format_datetime: format_datetime,
+        datetime_sort_helper: datetime_sort_helper,
+        dnd_contain_file: dnd_contain_file,
+        js_idx_to_char_idx: js_idx_to_char_idx,
+        char_idx_to_js_idx: char_idx_to_js_idx,
+        _ansispan:_ansispan,
+        change_favicon: change_favicon
     };
 
     return utils;
