@@ -15,7 +15,9 @@
 // * For dialogs, use a verb that indicates what the dialog will accomplish, such as "confirm-restart-kernel".
 
 
-define(function(require){
+define([
+    'base/js/i18n',
+    ], function(i18n){
     "use strict";
 
     var warn_bad_name = function(name){
@@ -34,8 +36,8 @@ define(function(require){
         Object.seal(this);
     };
 
-    var $ = require('jquery');
-    var events =  require('base/js/events');
+    var $ = requirejs('jquery');
+    var events =  requirejs('base/js/events');
 
     /**
      *  A bunch of predefined `Simple Actions` used by Jupyter.
@@ -64,13 +66,15 @@ define(function(require){
      **/
     var _actions = {
         'toggle-rtl-layout': {
-            help: 'Open a dialog to edit the command mode keyboard shortcuts',
+            cmd: i18n.msg._('toggle rtl layout'),
+            help: i18n.msg._('Toggle the screen directionality between left-to-right and right-to-left'),
             handler: function () {
               (document.body.getAttribute('dir')=='rtl') ? document.body.setAttribute('dir','ltr') : document.body.setAttribute('dir','rtl');
             }
         },
         'edit-command-mode-keyboard-shortcuts': {
-            help: 'Open a dialog to edit the command mode keyboard shortcuts',
+            cmd: i18n.msg._('edit command mode keyboard shortcuts'),
+            help: i18n.msg._('Open a dialog to edit the command mode keyboard shortcuts'),
             handler: function (env) {
                 env.notebook.show_shortcuts_editor();
             }
@@ -90,7 +94,8 @@ define(function(require){
             }
         },
         'restart-kernel': {
-            help: 'restart the kernel (no confirmation dialog)',
+            cmd: i18n.msg._('restart kernel'),
+            help: i18n.msg._('restart the kernel (no confirmation dialog)'),
             handler: function (env) {
                 env.notebook.restart_kernel({confirm: false});
             },
@@ -98,131 +103,156 @@ define(function(require){
         'confirm-restart-kernel':{
             icon: 'fa-repeat',
             help_index : 'hb',
-            help: 'restart the kernel (with dialog)',
+            cmd: i18n.msg._('confirm restart kernel'),
+            help: i18n.msg._('restart the kernel (with dialog)'),
             handler : function (env) {
                 env.notebook.restart_kernel();
             }
         },
         'restart-kernel-and-run-all-cells': {
-            help: 'restart the kernel, then re-run the whole notebook (no confirmation dialog)',
+            cmd: i18n.msg._('restart kernel and run all cells'),
+            help: i18n.msg._('restart the kernel, then re-run the whole notebook (no confirmation dialog)'),
             handler: function (env) {
                 env.notebook.restart_run_all({confirm: false});
             }
         },
         'confirm-restart-kernel-and-run-all-cells': {
-            help: 'restart the kernel, then re-run the whole notebook (with dialog)',
+            icon: 'fa-forward',
+            cmd: i18n.msg._('confirm restart kernel and run all cells'),
+            help: i18n.msg._('restart the kernel, then re-run the whole notebook (with dialog)'),
             handler: function (env) {
                 env.notebook.restart_run_all();
             }
         },
         'restart-kernel-and-clear-output': {
-            help: 'restart the kernel and clear all output (no confirmation dialog)',
+            cmd: i18n.msg._('restart kernel and clear output'),
+            help: i18n.msg._('restart the kernel and clear all output (no confirmation dialog)'),
             handler: function (env) {
                 env.notebook.restart_clear_output({confirm: false});
             }
         },
         'confirm-restart-kernel-and-clear-output': {
-            help: 'restart the kernel and clear all output (with dialog)',
+            cmd: i18n.msg._('confirm restart kernel and clear output'),
+            help: i18n.msg._('restart the kernel and clear all output (with dialog)'),
             handler: function (env) {
                 env.notebook.restart_clear_output();
             }
         },
         'interrupt-kernel':{
             icon: 'fa-stop',
+            cmd: i18n.msg._('interrupt the kernel'),
+            help: i18n.msg._('interrupt the kernel'),
             help_index : 'ha',
             handler : function (env) {
                 env.notebook.kernel.interrupt();
             }
         },
         'run-cell-and-select-next': {
+            cmd: i18n.msg._('run cell and select next'),
             icon: 'fa-step-forward',
-            help    : 'run cell, select below',
+            help: i18n.msg._('run cell, select below'),
             help_index : 'ba',
             handler : function (env) {
                 env.notebook.execute_cell_and_select_below();
             }
         },
         'run-cell':{
-            help    : 'run selected cells',
+            cmd: i18n.msg._('run selected cells'),
+            help    : i18n.msg._('run selected cells'),
             help_index : 'bb',
             handler : function (env) {
                 env.notebook.execute_selected_cells();
             }
         },
         'run-cell-and-insert-below':{
-            help    : 'run cell, insert below',
+            cmd: i18n.msg._('run cell and insert below'),
+            help    : i18n.msg._('run cell and insert below'),
             help_index : 'bc',
             handler : function (env) {
                 env.notebook.execute_cell_and_insert_below();
             }
         },
         'run-all-cells': {
-            help: 'run all cells',
+            cmd: i18n.msg._('run all cells'),
+            help: i18n.msg._('run all cells'),
             help_index: 'bd',
             handler: function (env) {
                 env.notebook.execute_all_cells();
             }
         },
         'run-all-cells-above':{
+            cmd: i18n.msg._('run all cells above'),
+            help: i18n.msg._('run all cells above'),
             handler : function (env) {
                 env.notebook.execute_cells_above();
             }
         },
         'run-all-cells-below':{
+            cmd: i18n.msg._('run all cells below'),
+            help: i18n.msg._('run all cells below'),
             handler : function (env) {
                 env.notebook.execute_cells_below();
             }
         },
         'enter-command-mode': {
-            help    : 'command mode',
+            cmd: i18n.msg._('enter command mode'),
+            help    : i18n.msg._('enter command mode'),
             help_index : 'aa',
             handler : function (env) {
                 env.notebook.command_mode();
             }
         },
         'insert-image': {
-            help      : 'insert image',
+            cmd: i18n.msg._('insert image'),
+            help      : i18n.msg._('insert image'),
             help_index : 'dz',
             handler : function (env) {
                 env.notebook.insert_image();
             }
         },
         'cut-cell-attachments': {
-            help    : 'cut cell attachments',
+            cmd: i18n.msg._('cut cell attachments'),
+            help    : i18n.msg._('cut cell attachments'),
             help_index : 'dza',
             handler: function (env) {
                 env.notebook.cut_cell_attachments();
             }
         },
         'copy-cell-attachments': {
-            help    : 'copy cell attachments',
+            cmd: i18n.msg._('copy cell attachments'),
+            help    : i18n.msg._('copy cell attachments'),
             help_index: 'dzb',
             handler: function (env) {
                 env.notebook.copy_cell_attachments();
             }
         },
         'paste-cell-attachments': {
-            help    : 'paste cell attachments',
+            cmd: i18n.msg._('paste cell attachments'),
+            help    : i18n.msg._('paste cell attachments'),
             help_index: 'dzc',
             handler: function (env) {
                 env.notebook.paste_cell_attachments();
             }
         },
         'split-cell-at-cursor': {
-            help    : 'split cell',
+            cmd: i18n.msg._('split cell at cursor'),
+            help    : i18n.msg._('split cell at cursor'),
             help_index : 'ea',
             handler : function (env) {
                 env.notebook.split_cell();
             }
         },
         'enter-edit-mode' : {
+            cmd: i18n.msg._('enter edit mode'),
+            help : i18n.msg._('enter edit mode'),
             help_index : 'aa',
             handler : function (env) {
                 env.notebook.edit_mode();
             }
         },
         'select-previous-cell' : {
-            help: 'select cell above',
+            cmd: i18n.msg._('select previous cell'),
+            help: i18n.msg._('select cell above'),
             help_index : 'da',
             handler : function (env) {
                 var index = env.notebook.get_selected_index();
@@ -233,7 +263,8 @@ define(function(require){
             }
         },
         'select-next-cell' : {
-            help: 'select cell below',
+            cmd: i18n.msg._('select next cell'),
+            help: i18n.msg._('select cell below'),
             help_index : 'db',
             handler : function (env) {
                 var index = env.notebook.get_selected_index();
@@ -244,29 +275,32 @@ define(function(require){
             }
         },
         'extend-selection-above' : {
-            help: 'extend selected cells above',
+            cmd: i18n.msg._('extend selection above'),
+            help: i18n.msg._('extend selected cells above'),
             help_index : 'dc',
             handler : function (env) {
                 env.notebook.extend_selection_by(-1);
-                // scroll into view, 
-                // do not call notebook.focus_cell(), or 
+                // scroll into view,
+                // do not call notebook.focus_cell(), or
                 // all the selection get thrown away
                 env.notebook.get_selected_cell().element.focus();
             }
         },
         'extend-selection-below' : {
-            help: 'extend selected cells below',
+            cmd: i18n.msg._('extend selection below'),
+            help: i18n.msg._('extend selected cells below'),
             help_index : 'dd',
             handler : function (env) {
                 env.notebook.extend_selection_by(1);
-                // scroll into view, 
-                // do not call notebook.focus_cell(), or 
+                // scroll into view,
+                // do not call notebook.focus_cell(), or
                 // all the selection get thrown away
                 env.notebook.get_selected_cell().element.focus();
             }
         },
         'cut-cell' : {
-            help: 'cut selected cells',
+            cmd: i18n.msg._('cut selected cells'),
+            help: i18n.msg._('cut selected cells'),
             icon: 'fa-cut',
             help_index : 'ee',
             handler : function (env) {
@@ -276,22 +310,31 @@ define(function(require){
             }
         },
         'copy-cell' : {
-            help: 'copy selected cells',
+            cmd: i18n.msg._('copy selected cells'),
+            help: i18n.msg._('copy selected cells'),
             icon: 'fa-copy',
             help_index : 'ef',
             handler : function (env) {
                 env.notebook.copy_cell();
             }
         },
+        'paste-cell-replace' : {
+            help: 'paste cells replace',
+            handler : function (env) {
+                env.notebook.paste_cell_replace();
+            }
+        },
         'paste-cell-above' : {
-            help: 'paste cells above',
+            cmd: i18n.msg._('paste cells above'),
+            help: i18n.msg._('paste cells above'),
             help_index : 'eg',
             handler : function (env) {
                 env.notebook.paste_cell_above();
             }
         },
         'paste-cell-below' : {
-            help: 'paste cells below',
+            cmd: i18n.msg._('paste cells below'),
+            help: i18n.msg._('paste cells below'),
             icon: 'fa-paste',
             help_index : 'eh',
             handler : function (env) {
@@ -299,7 +342,8 @@ define(function(require){
             }
         },
         'insert-cell-above' : {
-            help: 'insert cell above',
+            cmd: i18n.msg._('insert cell above'),
+            help: i18n.msg._('insert cell above'),
             help_index : 'ec',
             handler : function (env) {
                 env.notebook.insert_cell_above();
@@ -308,7 +352,8 @@ define(function(require){
             }
         },
         'insert-cell-below' : {
-            help: 'insert cell below',
+            cmd: i18n.msg._('insert cell below'),
+            help: i18n.msg._('insert cell below'),
             icon : 'fa-plus',
             help_index : 'ed',
             handler : function (env) {
@@ -318,90 +363,103 @@ define(function(require){
             }
         },
         'change-cell-to-code' : {
-            help    : 'to code',
+            cmd: i18n.msg._('change cell to code'),
+            help    : i18n.msg._('change cell to code'),
             help_index : 'ca',
             handler : function (env) {
                 env.notebook.cells_to_code();
             }
         },
         'change-cell-to-markdown' : {
-            help    : 'to markdown',
+            cmd: i18n.msg._('change cell to markdown'),
+            help    : i18n.msg._('change cell to markdown'),
             help_index : 'cb',
             handler : function (env) {
                 env.notebook.cells_to_markdown();
             }
         },
         'change-cell-to-raw' : {
-            help    : 'to raw',
+            cmd: i18n.msg._('change cell to raw'),
+            help    : i18n.msg._('change cell to raw'),
             help_index : 'cc',
             handler : function (env) {
                 env.notebook.cells_to_raw();
             }
         },
         'change-cell-to-heading-1' : {
-            help    : 'to heading 1',
+            cmd: i18n.msg._('change cell to heading 1'),
+            help    : i18n.msg._('change cell to heading 1'),
             help_index : 'cd',
             handler : function (env) {
                 env.notebook.to_heading(undefined, 1);
             }
         },
         'change-cell-to-heading-2' : {
-            help    : 'to heading 2',
+            cmd: i18n.msg._('change cell to heading 2'),
+            help    : i18n.msg._('change cell to heading 2'),
             help_index : 'ce',
             handler : function (env) {
                 env.notebook.to_heading(undefined, 2);
             }
         },
         'change-cell-to-heading-3' : {
-            help    : 'to heading 3',
+            cmd: i18n.msg._('change cell to heading 3'),
+            help    : i18n.msg._('change cell to heading 3'),
             help_index : 'cf',
             handler : function (env) {
                 env.notebook.to_heading(undefined, 3);
             }
         },
         'change-cell-to-heading-4' : {
-            help    : 'to heading 4',
+            cmd: i18n.msg._('change cell to heading 4'),
+            help    : i18n.msg._('change cell to heading 4'),
             help_index : 'cg',
             handler : function (env) {
                 env.notebook.to_heading(undefined, 4);
             }
         },
         'change-cell-to-heading-5' : {
-            help    : 'to heading 5',
+            cmd: i18n.msg._('change cell to heading 5'),
+            help    : i18n.msg._('change cell to heading 5'),
             help_index : 'ch',
             handler : function (env) {
                 env.notebook.to_heading(undefined, 5);
             }
         },
         'change-cell-to-heading-6' : {
-            help    : 'to heading 6',
+            cmd: i18n.msg._('change cell to heading 6'),
+            help    : i18n.msg._('change cell to heading 6'),
             help_index : 'ci',
             handler : function (env) {
                 env.notebook.to_heading(undefined, 6);
             }
         },
         'toggle-cell-output-collapsed' : {
-            help    : 'toggle output of selected cells',
+            cmd: i18n.msg._('toggle cell output'),
+            help    : i18n.msg._('toggle output of selected cells'),
             help_index : 'gb',
             handler : function (env) {
                 env.notebook.toggle_cells_outputs();
             }
         },
         'toggle-cell-output-scrolled' : {
-            help    : 'toggle output scrolling of selected cells',
+            cmd: i18n.msg._('toggle cell scrolling'),
+            help    : i18n.msg._('toggle output scrolling of selected cells'),
             help_index : 'gc',
             handler : function (env) {
                 env.notebook.toggle_cells_outputs_scroll();
             }
         },
         'clear-cell-output' : {
-            help    : 'clear output of selected cells',
+            cmd: i18n.msg._('clear cell output'),
+            help    : i18n.msg._('clear output of selected cells'),
             handler : function (env) {
                 env.notebook.clear_cells_outputs();
             }
         },
         'move-cell-down' : {
-            help: 'move selected cells down',
+            cmd: i18n.msg._('move cells down'),
+            help: i18n.msg._('move selected cells down'),
             icon: 'fa-arrow-down',
             help_index : 'eb',
             handler : function (env) {
@@ -409,7 +467,8 @@ define(function(require){
             }
         },
         'move-cell-up' : {
-            help: 'move selected cells up',
+            cmd: i18n.msg._('move cells up'),
+            help: i18n.msg._('move selected cells up'),
             icon: 'fa-arrow-up',
             help_index : 'ea',
             handler : function (env) {
@@ -417,54 +476,65 @@ define(function(require){
             }
         },
         'toggle-cell-line-numbers' : {
-            help    : 'toggle line numbers',
+            cmd: i18n.msg._('toggle line numbers'),
+            help    : i18n.msg._('toggle line numbers'),
             help_index : 'ga',
             handler : function (env) {
                 env.notebook.cell_toggle_line_numbers();
             }
         },
         'show-keyboard-shortcuts' : {
+            cmd: i18n.msg._('show keyboard shortcuts'),
+            help    : i18n.msg._('show keyboard shortcuts'),
             help_index : 'ge',
             handler : function (env) {
                 env.quick_help.show_keyboard_shortcuts();
             }
         },
         'delete-cell': {
-            help: 'delete selected cells',
+            cmd: i18n.msg._('delete cells'),
+            help: i18n.msg._('delete selected cells'),
             help_index : 'ej',
             handler : function (env) {
                 env.notebook.delete_cell();
             }
         },
         'undo-cell-deletion' : {
+            cmd: i18n.msg._('undo cell deletion'),
+            help: i18n.msg._('undo cell deletion'),
             help_index : 'ei',
             handler : function (env) {
                 env.notebook.undelete_cell();
             }
         },
         // TODO reminder
-        // open an issue, merge with above merge with last cell of notebook if at top. 
+        // open an issue, merge with above merge with last cell of notebook if at top.
         'merge-cell-with-previous-cell' : {
+            cmd: i18n.msg._('merge cell with previous cell'),
+            help    : i18n.msg._('merge cell above'),
             handler : function (env) {
                 env.notebook.merge_cell_above();
             }
         },
         'merge-cell-with-next-cell' : {
-            help    : 'merge cell below',
+            cmd: i18n.msg._('merge cell with next cell'),
+            help    : i18n.msg._('merge cell below'),
             help_index : 'ek',
             handler : function (env) {
                 env.notebook.merge_cell_below();
             }
         },
         'merge-selected-cells' : {
-            help : 'merge selected cells',
+            cmd: i18n.msg._('merge selected cells'),
+            help : i18n.msg._('merge selected cells'),
             help_index: 'el',
             handler: function(env) {
                 env.notebook.merge_selected_cells();
             }
         },
         'merge-cells' : {
-            help : 'merge selected cells, or current cell with cell below if only one cell selected',
+            cmd: i18n.msg._('merge cells'),
+            help : i18n.msg._('merge selected cells, or current cell with cell below if only one cell is selected'),
             help_index: 'el',
             handler: function(env) {
                 var l = env.notebook.get_selected_cells_indices().length;
@@ -477,14 +547,16 @@ define(function(require){
         },
         'show-command-palette': {
             help_index : 'aa',
-            help: 'open the command palette',
+            cmd: i18n.msg._('show command pallette'),
+            help: i18n.msg._('open the command palette'),
             icon: 'fa-keyboard-o',
             handler : function(env){
                 env.notebook.show_command_palette();
             }
         },
         'toggle-all-line-numbers': {
-            help : 'toggles line numbers in all cells, and persist the setting',
+            cmd: i18n.msg._('toggle all line numbers'),
+            help : i18n.msg._('toggles line numbers in all cells, and persist the setting'),
             icon: 'fa-list-ol',
             handler: function(env) {
                 var value = !env.notebook.line_numbers;
@@ -495,7 +567,8 @@ define(function(require){
             }
         },
         'show-all-line-numbers': {
-            help : 'show line numbers in all cells, and persist the setting',
+            cmd: i18n.msg._('show all line numbers'),
+            help : i18n.msg._('show line numbers in all cells, and persist the setting'),
             handler: function(env) {
                 env.notebook.get_cells().map(function(c) {
                     c.code_mirror.setOption('lineNumbers', true);
@@ -504,7 +577,8 @@ define(function(require){
             }
         },
         'hide-all-line-numbers': {
-            help : 'hide line numbers in all cells, and persist the setting',
+            cmd: i18n.msg._('hide all line numbers'),
+            help : i18n.msg._('hide line numbers in all cells, and persist the setting'),
             handler: function(env) {
                 env.notebook.get_cells().map(function(c) {
                     c.code_mirror.setOption('lineNumbers', false);
@@ -513,7 +587,8 @@ define(function(require){
             }
         },
         'toggle-header':{
-            help: 'hide/show the header',
+            cmd: i18n.msg._('toggle header'),
+            help: i18n.msg._('switch between showing and hiding the header'),
             handler : function(env) {
                 var value = !env.notebook.header;
                 if (value === true) {
@@ -528,7 +603,8 @@ define(function(require){
             }
         },
         'show-header':{
-            help: 'show the header',
+            cmd: i18n.msg._('show the header'),
+            help: i18n.msg._('show the header'),
             handler : function(env) {
                 $('#header-container').show();
                 $('.header-bar').show();
@@ -537,7 +613,8 @@ define(function(require){
             }
         },
         'hide-header':{
-            help: 'hide the header',
+            cmd: i18n.msg._('hide the header'),
+            help: i18n.msg._('hide the header'),
             handler : function(env) {
                 $('#header-container').hide();
                 $('.header-bar').hide();
@@ -545,8 +622,30 @@ define(function(require){
                 env.notebook.header = false;
             }
         },
+        'toggle-menubar':{
+            help: 'hide/show the menu bar',
+            handler : function(env) {
+                $('#menubar-container').toggle();
+                events.trigger('resize-header.Page');
+            }
+        },
+        'show-menubar':{
+            help: 'show the menu bar',
+            handler : function(env) {
+                $('#menubar-container').show();
+                events.trigger('resize-header.Page');
+            }
+        },
+        'hide-menubar':{
+            help: 'hide the menu bar',
+            handler : function(env) {
+                $('#menubar-container').hide();
+                events.trigger('resize-header.Page');
+            }
+        },
         'toggle-toolbar':{
-            help: 'hide/show the toolbar',
+            cmd: i18n.msg._('toggle toolbar'),
+            help: i18n.msg._('switch between showing and hiding the toolbar'),
             handler : function(env) {
                 var value = !env.notebook.toolbar;
                 if (value === true) {
@@ -559,7 +658,8 @@ define(function(require){
             }
         },
         'show-toolbar':{
-            help: 'show the toolbar',
+            cmd: i18n.msg._('show the toolbar'),
+            help: i18n.msg._('show the toolbar'),
             handler : function(env) {
                 $('div#maintoolbar').show();
                 events.trigger('resize-header.Page');
@@ -567,7 +667,8 @@ define(function(require){
             }
         },
         'hide-toolbar':{
-            help: 'hide the toolbar',
+            cmd: i18n.msg._('hide the toolbar'),
+            help: i18n.msg._('hide the toolbar'),
             handler : function(env) {
                 $('div#maintoolbar').hide();
                 events.trigger('resize-header.Page');
@@ -575,7 +676,8 @@ define(function(require){
             }
         },
         'close-pager': {
-            help : 'close the pager',
+            cmd: i18n.msg._('close the pager'),
+            help : i18n.msg._('close the pager'),
             handler : function(env) {
                 // Collapse the page if it is open
                 if (env.pager && env.pager.expanded) {
@@ -583,6 +685,16 @@ define(function(require){
                 }
             }
         },
+        'auto-indent': {
+            cmd: i18n.msg._('automatically indent selection'),
+            help : i18n.msg._('automatically indent selection'),
+            handler : function(env) {
+              // Get selected cell
+              var selected_cell = env.notebook.get_selected_cell();
+              // Execute a CM command
+              selected_cell.code_mirror.execCommand('indentAuto');
+            }
+        }
     };
 
     /**
@@ -600,11 +712,14 @@ define(function(require){
      **/
     var custom_ignore = {
         'ignore':{
+            cmd: i18n.msg._('ignore'),
             handler : function () {
                 return true;
             }
         },
         'move-cursor-up':{
+            cmd: i18n.msg._('move cursor up'),
+            help: i18n.msg._("move cursor up"),
             handler : function (env, event) {
                 var index = env.notebook.get_selected_index();
                 var cell = env.notebook.get_cell(index);
@@ -624,6 +739,8 @@ define(function(require){
             }
         },
         'move-cursor-down':{
+            cmd: i18n.msg._('move cursor down'),
+            help: i18n.msg._("move cursor down"),
             handler : function (env, event) {
                 var index = env.notebook.get_selected_index();
                 var cell = env.notebook.get_cell(index);
@@ -641,6 +758,8 @@ define(function(require){
             }
         },
         'scroll-notebook-down': {
+            cmd: i18n.msg._('scroll notebook down'),
+            help: i18n.msg._("scroll notebook down"),
             handler: function(env, event) {
                 if(event){
                     event.preventDefault();
@@ -649,6 +768,8 @@ define(function(require){
             },
         },
         'scroll-notebook-up': {
+            cmd: i18n.msg._('scroll notebook up'),
+            help: i18n.msg._("scroll notebook up"),
             handler: function(env, event) {
                 if(event){
                     event.preventDefault();
@@ -657,7 +778,8 @@ define(function(require){
             },
         },
         'scroll-cell-center': {
-            help: "Scroll the current cell to the center",
+            cmd: i18n.msg._('scroll cell center'),
+            help: i18n.msg._("Scroll the current cell to the center"),
             handler: function (env, event) {
                 if(event){
                     event.preventDefault();
@@ -667,7 +789,8 @@ define(function(require){
             }
         },
         'scroll-cell-top': {
-            help: "Scroll the current cell to the top",
+            cmd: i18n.msg._('scroll cell top'),
+            help: i18n.msg._("Scroll the current cell to the top"),
             handler: function (env, event) {
                 if(event){
                     event.preventDefault();
@@ -677,44 +800,51 @@ define(function(require){
             }
         },
         'duplicate-notebook':{
-            help: "Create an open a copy of current notebook",
+            cmd: i18n.msg._('duplicate notebook'),
+            help: i18n.msg._("Create and open a copy of the current notebook"),
             handler : function (env, event) {
                 env.notebook.copy_notebook();
             }
         },
         'trust-notebook':{
-            help: "Trust the current notebook",
+            cmd: i18n.msg._('trust notebook'),
+            help: i18n.msg._("Trust the current notebook"),
             handler : function (env, event) {
                 env.notebook.trust_notebook();
             }
         },
         'rename-notebook':{
-            help: "Rename current notebook",
+            cmd: i18n.msg._('rename notebook'),
+            help: i18n.msg._("Rename the current notebook"),
             handler : function (env, event) {
                 env.notebook.save_widget.rename_notebook({notebook: env.notebook});
             }
         },
         'toggle-all-cells-output-collapsed':{
-            help: "Toggle the hiddens state of all output areas",
+            cmd: i18n.msg._('toggle all cells output collapsed'),
+            help: i18n.msg._("Toggle the hidden state of all output areas"),
             handler : function (env, event) {
                 env.notebook.toggle_all_output();
             }
         },
         'toggle-all-cells-output-scrolled':{
-            help: "Toggle the scrolling state of all output areas",
+            cmd: i18n.msg._('toggle all cells output scrolled'),
+            help: i18n.msg._("Toggle the scrolling state of all output areas"),
             handler : function (env, event) {
                 env.notebook.toggle_all_output_scroll();
             }
         },
 
         'clear-all-cells-output':{
-            help: "Clear the content of all the outputs",
+            cmd: i18n.msg._('clear all cells output'),
+            help: i18n.msg._("Clear the content of all the outputs"),
             handler : function (env, event) {
                 env.notebook.clear_all_output();
             }
         },
         'save-notebook':{
-            help: "Save and Checkpoint",
+            cmd: i18n.msg._('save notebook'),
+            help: i18n.msg._("Save and Checkpoint"),
             help_index : 'fb',
             icon: 'fa-save',
             handler : function (env, event) {
@@ -731,6 +861,7 @@ define(function(require){
     // and uniformize/fill in missing pieces in of an action.
     var _prepare_handler = function(registry, subkey, source){
         registry['jupyter-notebook:'+subkey] = {};
+        registry['jupyter-notebook:'+subkey].cmd = source[subkey].cmd;
         registry['jupyter-notebook:'+subkey].help = source[subkey].help||subkey.replace(/-/g,' ');
         registry['jupyter-notebook:'+subkey].help_index = source[subkey].help_index;
         registry['jupyter-notebook:'+subkey].icon = source[subkey].icon;
@@ -829,12 +960,12 @@ define(function(require){
 
     ActionHandler.prototype.get_name = function(name_or_data){
         /**
-         * given an `action` or `name` of a action, return the name attached to this action.
+         * given an `action` or `name` of an action, return the name attached to this action.
          * if given the name of and corresponding actions does not exist in registry, return `null`.
          **/
 
         if(typeof(name_or_data) === 'string'){
-            warn_bad_name(name);
+            warn_bad_name(name_or_data);
             if(this.exists(name_or_data)){
                 return name_or_data;
             } else {
