@@ -1,11 +1,13 @@
 define([
     'jquery',
     'base/js/utils',
+    'base/js/i18n',
     'base/js/dialog',
     'base/js/notificationarea',
     'moment'
-], function($, utils, dialog, notificationarea, moment) {
+], function($, utils, i18n, dialog, notificationarea, moment) {
     "use strict";
+
     var NotificationArea = notificationarea.NotificationArea;
     
     var NotebookNotificationArea = function(selector, options) {
@@ -40,6 +42,22 @@ define([
         var $modal_ind_icon = $("#modal_indicator");
         var $readonly_ind_icon = $('#readonly-indicator');
         var $body = $('body');
+        var busy_favicon_timer = -1;
+        
+        var set_busy_favicon = function(on) {
+            if (on) {
+                // Only show the busy icon if execution lasts > 1s
+                // This is to avoid rapidly switching icons and making lots of
+                // HTTP requests.
+                clearTimeout(busy_favicon_timer);
+                busy_favicon_timer = setTimeout(function() {
+                    utils.change_favicon('/static/base/images/favicon-busy-1.ico');
+                }, 1000);
+            } else {
+                clearTimeout(busy_favicon_timer);
+                utils.change_favicon('/static/base/images/favicon-notebook.ico');
+            }
+        };
 
         // Listen for the notebook loaded event.  Set readonly indicator.
         this.events.on('notebook_loaded.Notebook', function() {
@@ -55,45 +73,50 @@ define([
             that.save_widget.update_document_title();
             $body.addClass('edit_mode');
             $body.removeClass('command_mode');
-            $modal_ind_icon.attr('title','Edit Mode');
+            $modal_ind_icon.attr('title',i18n.msg._('Edit Mode'));
         });
 
         this.events.on('command_mode.Notebook', function () {
             that.save_widget.update_document_title();
             $body.removeClass('edit_mode');
             $body.addClass('command_mode');
-            $modal_ind_icon.attr('title','Command Mode');
+            $modal_ind_icon.attr('title',i18n.msg._('Command Mode'));
         });
 
         // Implicitly start off in Command mode, switching to Edit mode will trigger event
-        $modal_ind_icon.addClass('modal_indicator').attr('title','Command Mode');
+        $modal_ind_icon.addClass('modal_indicator').attr('title',i18n.msg._('Command Mode'));
         $body.addClass('command_mode');
 
         // Kernel events
 
         // this can be either kernel_created.Kernel or kernel_created.Session
         this.events.on('kernel_created.Kernel kernel_created.Session', function () {
-            knw.info("Kernel Created", 500);
+            knw.info(i18n.msg._("Kernel Created"), 500);
         });
 
         this.events.on('kernel_reconnecting.Kernel', function () {
-            knw.warning("Connecting to kernel");
+            knw.warning(i18n.msg._("Connecting to kernel"));
         });
 
         this.events.on('kernel_connection_dead.Kernel', function (evt, info) {
-            knw.danger("Not Connected", undefined, function () {
+            knw.danger(i18n.msg._("Not Connected"), undefined, function () {
                 // schedule reconnect a short time in the future, don't reconnect immediately
                 setTimeout($.proxy(info.kernel.reconnect, info.kernel), 500);
-            }, {title: 'click to reconnect'});
+            }, {title: i18n.msg._('click to reconnect')});
         });
 
         this.events.on('kernel_connected.Kernel', function () {
             knw.info("Connected", 500);
+            // trigger busy in the status to clear broken-link state immediately
+            // a kernel_ready event will come when the kernel becomes responsive.
+            $kernel_ind_icon
+                .attr('class', 'kernel_busy_icon')
+                .attr('title', i18n.msg._('Kernel Connected'));
         });
 
         this.events.on('kernel_restarting.Kernel', function () {
             that.save_widget.update_document_title();
-            knw.set_message("Restarting kernel", 2000);
+            knw.set_message(i18n.msg._("Restarting kernel"), 2000);
         });
 
         this.events.on('kernel_autorestarting.Kernel', function (evt, info) {
@@ -107,8 +130,8 @@ define([
                 dialog.kernel_modal({
                     notebook: that.notebook,
                     keyboard_manager: that.keyboard_manager,
-                    title: "Kernel Restarting",
-                    body: "The kernel appears to have died. It will restart automatically.",
+                    title: i18n.msg._("Kernel Restarting"),
+                    body: i18n.msg._("The kernel appears to have died. It will restart automatically."),
                     buttons: {
                         OK : {
                             class : "btn-primary"
@@ -118,18 +141,18 @@ define([
             }
 
             that.save_widget.update_document_title();
-            knw.danger("Dead kernel");
-            $kernel_ind_icon.attr('class','kernel_dead_icon').attr('title','Kernel Dead');
+            knw.danger(i18n.msg._("Dead kernel"));
+            $kernel_ind_icon.attr('class','kernel_dead_icon').attr('title',i18n.msg._('Kernel Dead'));
         });
 
         this.events.on('kernel_interrupting.Kernel', function () {
-            knw.set_message("Interrupting kernel", 2000);
+            knw.set_message(i18n.msg._("Interrupting kernel"), 2000);
         });
 
         this.events.on('kernel_disconnected.Kernel', function () {
             $kernel_ind_icon
                 .attr('class', 'kernel_disconnected_icon')
-                .attr('title', 'No Connection to Kernel');
+                .attr('title', i18n.msg._('No Connection to Kernel'));
         });
 
         this.events.on('kernel_connection_failed.Kernel', function (evt, info) {
@@ -139,12 +162,12 @@ define([
             // with messages
             if (info.attempt === 1) {
 
-                var msg = "A connection to the notebook server could not be established." +
+                var msg = i18n.msg._("A connection to the notebook server could not be established." +
                         " The notebook will continue trying to reconnect. Check your" +
-                        " network connection or notebook server configuration.";
+                        " network connection or notebook server configuration.");
 
-                dialog.kernel_modal({
-                    title: "Connection failed",
+                var the_dialog = dialog.kernel_modal({
+                    title: i18n.msg._("Connection failed"),
                     body: msg,
                     keyboard_manager: that.keyboard_manager,
                     notebook: that.notebook,
@@ -152,27 +175,41 @@ define([
                         "OK": {}
                     }
                 });
+
+                // hide the dialog on reconnect if it's still visible
+                var dismiss = function () {
+                    the_dialog.modal('hide');
+                }
+                that.events.on("kernel_connected.Kernel", dismiss);
+                the_dialog.on("hidden.bs.modal", function () {
+                    // clear handler on dismiss
+                    that.events.off("kernel_connected.Kernel", dismiss);
+                });
             }
         });
 
         this.events.on('kernel_killed.Kernel kernel_killed.Session', function () {
             that.save_widget.update_document_title();
-            knw.warning("No kernel");
-            $kernel_ind_icon.attr('class','kernel_busy_icon').attr('title','Kernel is not running');
+            knw.warning(i18n.msg._("No kernel"));
+            $kernel_ind_icon.attr('class','kernel_busy_icon').attr('title',i18n.msg._('Kernel is not running'));
         });
 
         this.events.on('kernel_dead.Kernel', function () {
+        // This statement is used simply so that message extraction
+        // will pick up the strings.  The actual setting of the text
+        // for the button is in dialog.js.
+        var button_labels = [ i18n.msg._("Don't Restart"), i18n.msg._("Try Restarting Now"), i18n.msg._("OK")];
 
             var showMsg = function () {
 
-                var msg = 'The kernel has died, and the automatic restart has failed.' +
-                        ' It is possible the kernel cannot be restarted.' +
-                        ' If you are not able to restart the kernel, you will still be able to save' +
+                var msg = i18n.msg._('The kernel has died, and the automatic restart has failed.' +
+                        ' It is possible the kernel cannot be restarted. ' +
+                        'If you are not able to restart the kernel, you will still be able to save' +
                         ' the notebook, but running code will no longer work until the notebook' +
-                        ' is reopened.';
+                        ' is reopened.');
 
                 dialog.kernel_modal({
-                    title: "Dead kernel",
+                    title: i18n.msg._("Dead kernel"),
                     body : msg,
                     keyboard_manager: that.keyboard_manager,
                     notebook: that.notebook,
@@ -192,14 +229,14 @@ define([
             };
 
             that.save_widget.update_document_title();
-            knw.danger("Dead kernel", undefined, showMsg);
-            $kernel_ind_icon.attr('class','kernel_dead_icon').attr('title','Kernel Dead');
+            knw.danger(i18n.msg._("Dead kernel"), undefined, showMsg);
+            $kernel_ind_icon.attr('class','kernel_dead_icon').attr('title',i18n.msg._('Kernel Dead'));
 
             showMsg();
         });
         
         this.events.on("no_kernel.Kernel", function (evt, data) {
-            $("#kernel_indicator").find('.kernel_indicator_name').text("No Kernel");
+            $("#kernel_indicator").find('.kernel_indicator_name').text(i18n.msg._("No Kernel"));
         });
 
         this.events.on('kernel_dead.Session', function (evt, info) {
@@ -226,13 +263,13 @@ define([
                 }
 
                 dialog.kernel_modal({
-                    title: "Failed to start the kernel",
+                    title: i18n.msg._("Failed to start the kernel"),
                     body : msg,
                     keyboard_manager: that.keyboard_manager,
                     notebook: that.notebook,
                     open: cm_open,
                     buttons : {
-                        "Ok": { class: 'btn-primary' }
+                        "OK": { class: 'btn-primary' }
                     }
                 });
 
@@ -240,57 +277,47 @@ define([
             };
 
             that.save_widget.update_document_title();
-            $kernel_ind_icon.attr('class','kernel_dead_icon').attr('title','Kernel Dead');
+            $kernel_ind_icon.attr('class','kernel_dead_icon').attr('title',i18n.msg._('Kernel Dead'));
             knw.danger(short, undefined, showMsg);
         });
         
-        var change_favicon = function (src) {
-            var link = document.createElement('link'),
-                oldLink = document.getElementById('favicon');
-            link.id = 'favicon';
-            link.type = 'image/x-icon';
-            link.rel = 'shortcut icon';
-            link.href = src;
-            if (oldLink) document.head.removeChild(oldLink);
-            document.head.appendChild(link);
-        };
-
         this.events.on('kernel_starting.Kernel kernel_created.Session', function () {
             // window.document.title='(Starting) '+window.document.title;
-            $kernel_ind_icon.attr('class','kernel_busy_icon').attr('title','Kernel Busy');
-            knw.set_message("Kernel starting, please wait...");
-            change_favicon('/static/base/images/favicon-busy.ico');
+            $kernel_ind_icon.attr('class','kernel_busy_icon').attr('title',i18n.msg._('Kernel Busy'));
+            knw.set_message(i18n.msg._("Kernel starting, please wait..."));
+            set_busy_favicon(true);
         });
 
         this.events.on('kernel_ready.Kernel', function () {
             // that.save_widget.update_document_title();
-            $kernel_ind_icon.attr('class','kernel_idle_icon').attr('title','Kernel Idle');
-            knw.info("Kernel ready", 500);
-            change_favicon('/static/base/images/favicon.ico');
+            $kernel_ind_icon.attr('class','kernel_idle_icon').attr('title',i18n.msg._('Kernel Idle'));
+            knw.info(i18n.msg._("Kernel ready"), 500);
+            set_busy_favicon(false);
         });
 
         this.events.on('kernel_idle.Kernel', function () {
             // that.save_widget.update_document_title();
-            $kernel_ind_icon.attr('class','kernel_idle_icon').attr('title','Kernel Idle');
-            change_favicon('/static/base/images/favicon.ico');
+            $kernel_ind_icon.attr('class','kernel_idle_icon').attr('title',i18n.msg._('Kernel Idle'));
+            set_busy_favicon(false);
         });
 
         this.events.on('kernel_busy.Kernel', function () {
             // window.document.title='(Busy) '+window.document.title;
-            $kernel_ind_icon.attr('class','kernel_busy_icon').attr('title','Kernel Busy');
-            change_favicon('/static/base/images/favicon-busy.ico');
+            $kernel_ind_icon.attr('class','kernel_busy_icon').attr('title',i18n.msg._('Kernel Busy'));
+            set_busy_favicon(true);
         });
 
         this.events.on('spec_match_found.Kernel', function (evt, data) {
-            that.widget('kernelspec').info("Using kernel: " + data.found.spec.display_name, 3000, undefined, {
-                title: "Only candidate for language: " + data.selected.language + " was " + data.found.spec.display_name
+            that.widget('kernelspec').info(i18n.msg._("Using kernel: ") + data.found.spec.display_name, 3000, undefined, {
+                title: i18n.msg.sprintf(i18n.msg._("Only candidate for language: %1$s was %2$s."),
+                        data.selected.language, data.found.spec.display_name)
             });
         });
 
         
         // Start the kernel indicator in the busy state, and send a kernel_info request.
         // When the kernel_info reply arrives, the kernel is idle.
-        $kernel_ind_icon.attr('class','kernel_busy_icon').attr('title','Kernel Busy');
+        $kernel_ind_icon.attr('class','kernel_busy_icon').attr('title',i18n.msg._('Kernel Busy'));
     };
 
     /**
@@ -303,27 +330,27 @@ define([
 
         // Notebook events
         this.events.on('notebook_loading.Notebook', function () {
-            nnw.set_message("Loading notebook",500);
+            nnw.set_message(i18n.msg._("Loading notebook"),500);
         });
         this.events.on('notebook_loaded.Notebook', function () {
-            nnw.set_message("Notebook loaded",500);
+            nnw.set_message(i18n.msg._("Notebook loaded"),500);
         });
         this.events.on('notebook_saving.Notebook', function () {
-            nnw.set_message("Saving notebook",500);
+            nnw.set_message(i18n.msg._("Saving notebook"),500);
         });
         this.events.on('notebook_saved.Notebook', function () {
-            nnw.set_message("Notebook saved",2000);
+            nnw.set_message(i18n.msg._("Notebook saved"),2000);
         });
         this.events.on('notebook_save_failed.Notebook', function (evt, error) {
-            nnw.warning(error.message || "Notebook save failed");
+            nnw.warning(error.message || i18n.msg._("Notebook save failed"));
         });
         this.events.on('notebook_copy_failed.Notebook', function (evt, error) {
-            nnw.warning(error.message || "Notebook copy failed");
+            nnw.warning(error.message || i18n.msg._("Notebook copy failed"));
         });
         
         // Checkpoint events
         this.events.on('checkpoint_created.Notebook', function (evt, data) {
-            var msg = "Checkpoint created";
+            var msg = i18n.msg._("Checkpoint created");
             if (data.last_modified) {
                 var d = new Date(data.last_modified);
                 msg = msg + ": " + moment(d).format("HH:mm:ss");
@@ -331,27 +358,27 @@ define([
             nnw.set_message(msg, 2000);
         });
         this.events.on('checkpoint_failed.Notebook', function () {
-            nnw.warning("Checkpoint failed");
+            nnw.warning(i18n.msg._("Checkpoint failed"));
         });
         this.events.on('checkpoint_deleted.Notebook', function () {
-            nnw.set_message("Checkpoint deleted", 500);
+            nnw.set_message(i18n.msg._("Checkpoint deleted"), 500);
         });
         this.events.on('checkpoint_delete_failed.Notebook', function () {
-            nnw.warning("Checkpoint delete failed");
+            nnw.warning(i18n.msg._("Checkpoint delete failed"));
         });
         this.events.on('checkpoint_restoring.Notebook', function () {
-            nnw.set_message("Restoring to checkpoint...", 500);
+            nnw.set_message(i18n.msg._("Restoring to checkpoint..."), 500);
         });
         this.events.on('checkpoint_restore_failed.Notebook', function () {
-            nnw.warning("Checkpoint restore failed");
+            nnw.warning(i18n.msg._("Checkpoint restore failed"));
         });
 
         // Autosave events
         this.events.on('autosave_disabled.Notebook', function () {
-            nnw.set_message("Autosave disabled", 2000);
+            nnw.set_message(i18n.msg._("Autosave disabled"), 2000);
         });
         this.events.on('autosave_enabled.Notebook', function (evt, interval) {
-            nnw.set_message("Saving every " + interval / 1000 + "s", 1000);
+            nnw.set_message(i18n.msg.sprintf(i18n.msg._("Saving every %d sec."), interval / 1000) , 1000);
         });
     };
 
@@ -367,9 +394,9 @@ define([
         // Notebook trust events
         this.events.on('trust_changed.Notebook', function (event, trusted) {
             if (trusted) {
-                tnw.set_message("Trusted");
+                tnw.set_message(i18n.msg._("Trusted"));
             } else {
-                tnw.set_message("Not Trusted", undefined, function() {
+                tnw.set_message(i18n.msg._("Not Trusted"), undefined, function() {
                   that.notebook.trust_notebook();
                   return false;
                 });
