@@ -1,37 +1,42 @@
 // Copyright (c) Jupyter Development Team.
 // Distributed under the terms of the Modified BSD License.
 
-require([
+requirejs([
     'jquery',
-    'termjs',
     'base/js/utils',
     'base/js/page',
+    'auth/js/loginwidget',
     'services/config',
     'terminal/js/terminado',
-    'custom',
 ], function(
-    $, 
-    termjs,
+    $,
     utils,
     page,
+    loginwidget,
     configmod,
     terminado
     ){
     "use strict";
-    page = new page.Page();
+    requirejs(['custom/custom'], function() {});
+    page = new page.Page('div#header', 'div#site');
 
-    var config = new configmod.ConfigSection('terminal',
-                                     {base_url: utils.get_body_data('baseUrl')});
+    var common_options = {
+        base_url : utils.get_body_data("baseUrl"),
+    };
+
+    var config = new configmod.ConfigSection('terminal', common_options);
     config.load();
-    var common_config = new configmod.ConfigSection('common', 
-                                    {base_url: utils.get_body_data('baseUrl')});
+    var common_config = new configmod.ConfigSection('common', common_options);
     common_config.load();
+
+    var login_widget = new loginwidget.LoginWidget('span#login_widget', common_options);
+
     // Test size: 25x80
     var termRowHeight = function(){ return 1.00 * $("#dummy-screen")[0].offsetHeight / 25;};
         // 1.02 here arrived at by trial and error to make the spacing look right
     var termColWidth =  function() { return 1.02 * $("#dummy-screen-rows")[0].offsetWidth / 80;};
 
-    var base_url = utils.get_body_data('baseUrl');
+    var base_url = utils.get_body_data('baseUrl').replace(/\/?$/, '/');
     var ws_path = utils.get_body_data('wsPath');
     var ws_url = utils.get_body_data('wsUrl');
     if (!ws_url) {
@@ -41,6 +46,7 @@ require([
     ws_url = ws_url + base_url + ws_path;
     
     var header = $("#header")[0];
+
     function calculate_size() {
         var height = $(window).height() - header.offsetHeight;
         var width = $('#terminado-container').width();
