@@ -3,11 +3,12 @@
 from __future__ import print_function
 
 from binascii import hexlify
-import os
-import time
-import requests
 from contextlib import contextmanager
+import errno
+import os
+import sys
 from threading import Thread, Event
+import time
 from unittest import TestCase
 
 pjoin = os.path.join
@@ -17,10 +18,12 @@ try:
 except ImportError:
     from mock import patch #py2
 
+import requests
 from tornado.ioloop import IOLoop
 import zmq
 
 import jupyter_core.paths
+from traitlets.config import Config
 from ..notebookapp import NotebookApp
 from ..utils import url_path_join
 from ipython_genutils.tempdir import TemporaryDirectory
@@ -35,7 +38,7 @@ class TimeoutError(Exception):
 
 class NotebookTestBase(TestCase):
     """A base class for tests that need a running notebook.
-    
+
     This create some empty config and runtime directories
     and then starts the notebook server with them.
     """
@@ -61,7 +64,7 @@ class NotebookTestBase(TestCase):
                 return
 
         raise TimeoutError("The notebook server didn't start up correctly.")
-    
+
     @classmethod
     def wait_until_dead(cls):
         """Wait for the server process to terminate after shutdown"""
@@ -70,49 +73,79 @@ class NotebookTestBase(TestCase):
             raise TimeoutError("Undead notebook server")
     
     @classmethod
-    def request(self, verb, path, **kwargs):
+    def auth_headers(cls):
+        headers = {}
+        if cls.token:
+            headers['Authorization'] = 'token %s' % cls.token
+        return headers
+
+    @classmethod
+    def request(cls, verb, path, **kwargs):
         """Send a request to my server
         
         with authentication and everything.
         """
         headers = kwargs.setdefault('headers', {})
-        # kwargs.setdefault('allow_redirects', False)
-        headers.setdefault('Authorization', 'token %s' % self.token)
+        headers.update(cls.auth_headers())
         response = requests.request(verb,
-            url_path_join(self.base_url(), path),
+            url_path_join(cls.base_url(), path),
             **kwargs)
         return response
-
+    
     @classmethod
     def setup_class(cls):
-        cls.home_dir = TemporaryDirectory()
-        data_dir = TemporaryDirectory()
+        cls.tmp_dir = TemporaryDirectory()
+        def tmp(*parts):
+            path = os.path.join(cls.tmp_dir.name, *parts)
+            try:
+                os.makedirs(path)
+            except OSError as e:
+                if e.errno != errno.EEXIST:
+                    raise
+            return path
+        
+        cls.home_dir = tmp('home')
+        data_dir = cls.data_dir = tmp('data')
+        config_dir = cls.config_dir = tmp('config')
+        runtime_dir = cls.runtime_dir = tmp('runtime')
+        cls.notebook_dir = tmp('notebooks')
         cls.env_patch = patch.dict('os.environ', {
-            'HOME': cls.home_dir.name,
-            'IPYTHONDIR': pjoin(cls.home_dir.name, '.ipython'),
-            'JUPYTER_DATA_DIR' : data_dir.name
+            'HOME': cls.home_dir,
+            'PYTHONPATH': os.pathsep.join(sys.path),
+            'IPYTHONDIR': pjoin(cls.home_dir, '.ipython'),
+            'JUPYTER_NO_CONFIG': '1', # needed in the future
+            'JUPYTER_CONFIG_DIR' : config_dir,
+            'JUPYTER_DATA_DIR' : data_dir,
+            'JUPYTER_RUNTIME_DIR': runtime_dir,
         })
         cls.env_patch.start()
-        cls.path_patch = patch.object(jupyter_core.paths, 'SYSTEM_JUPYTER_PATH', [])
+        cls.path_patch = patch.multiple(
+            jupyter_core.paths,
+            SYSTEM_JUPYTER_PATH=[tmp('share', 'jupyter')],
+            ENV_JUPYTER_PATH=[tmp('env', 'share', 'jupyter')],
+            SYSTEM_CONFIG_PATH=[tmp('etc', 'jupyter')],
+            ENV_CONFIG_PATH=[tmp('env', 'etc', 'jupyter')],
+        )
         cls.path_patch.start()
-        cls.config_dir = TemporaryDirectory()
-        cls.data_dir = data_dir
-        cls.runtime_dir = TemporaryDirectory()
-        cls.notebook_dir = TemporaryDirectory()
+
+        config = cls.config or Config()
+        config.NotebookNotary.db_file = ':memory:'
+
         cls.token = hexlify(os.urandom(4)).decode('ascii')
-        
+
         started = Event()
         def start_thread():
             app = cls.notebook = NotebookApp(
                 port=cls.port,
                 port_retries=0,
                 open_browser=False,
-                config_dir=cls.config_dir.name,
-                data_dir=cls.data_dir.name,
-                runtime_dir=cls.runtime_dir.name,
-                notebook_dir=cls.notebook_dir.name,
+                config_dir=cls.config_dir,
+                data_dir=cls.data_dir,
+                runtime_dir=cls.runtime_dir,
+                notebook_dir=cls.notebook_dir,
                 base_url=cls.url_prefix,
-                config=cls.config,
+                config=config,
+                allow_root=True,
                 token=cls.token,
             )
             # don't register signal handler during tests
@@ -133,6 +166,7 @@ class NotebookTestBase(TestCase):
                 started.set()
                 app.session_manager.close()
         cls.notebook_thread = Thread(target=start_thread)
+        cls.notebook_thread.daemon = True
         cls.notebook_thread.start()
         started.wait()
         cls.wait_until_alive()
@@ -141,11 +175,7 @@ class NotebookTestBase(TestCase):
     def teardown_class(cls):
         cls.notebook.stop()
         cls.wait_until_dead()
-        cls.home_dir.cleanup()
-        cls.config_dir.cleanup()
-        cls.data_dir.cleanup()
-        cls.runtime_dir.cleanup()
-        cls.notebook_dir.cleanup()
+        cls.tmp_dir.cleanup()
         cls.env_patch.stop()
         cls.path_patch.stop()
         # cleanup global zmq Context, to ensure we aren't leaving dangling sockets
@@ -175,4 +205,4 @@ def assert_http_error(status, msg=None):
         if msg:
             assert msg in str(e), e
     else:
-        assert False, "Expected HTTP error status"
\ No newline at end of file
+        assert False, "Expected HTTP error status"
